@@ -1,25 +1,13 @@
-/*******************************************************************************
+/******************************************************************************
+  This file is part of PyTango (http://www.tinyurl.com/PyTango)
 
-   This file is part of PyTango, a python binding for Tango
+  Copyright 2006-2012 CELLS / ALBA Synchrotron, Bellaterra, Spain
+  Copyright 2013-2014 European Synchrotron Radiation Facility, Grenoble, France
 
-   http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
-
-   Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-   
-   PyTango is free software: you can redistribute it and/or modify
-   it under the terms of the GNU Lesser General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
-   PyTango is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Lesser General Public License for more details.
-  
-   You should have received a copy of the GNU Lesser General Public License
-   along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
-   
-*******************************************************************************/
+  Distributed under the terms of the GNU Lesser General Public License,
+  either version 3 of the License, or (at your option) any later version.
+  See LICENSE.txt for more info.
+******************************************************************************/
 
 #include "precompiled_header.hpp"
 #include "device_attribute.h"
@@ -424,10 +412,10 @@ void export_device_proxy()
         .def("get_device_db", &Tango::DeviceProxy::get_device_db,
             bopy::return_value_policy<bopy::reference_existing_object>())
 
-        .def("status", &PyDeviceProxy::status,
+        .def("_status", &PyDeviceProxy::status,
             ( arg_("self") ) )
 
-        .def("state", &PyDeviceProxy::state,
+        .def("_state", &PyDeviceProxy::state,
             ( arg_("self") ) )
 
         .def("adm_name", &Tango::DeviceProxy::adm_name,
@@ -442,7 +430,10 @@ void export_device_proxy()
         .def("alias", &Tango::DeviceProxy::alias,
             ( arg_("self") ) )
 
-        .def("ping", &PyDeviceProxy::ping,
+        .def("get_tango_lib_version", &Tango::DeviceProxy::get_tango_lib_version,
+            ( arg_("self") ) )        
+
+        .def("_ping", &PyDeviceProxy::ping,
             ( arg_("self") ) )
             
 
@@ -550,21 +541,21 @@ void export_device_proxy()
             &PyDeviceProxy::read_attribute,
             ( arg_("self"), arg_("attr_name"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
-        .def("read_attributes",
+        .def("_read_attributes",
             &PyDeviceProxy::read_attributes,
             ( arg_("self"), arg_("attr_names"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
-        .def("write_attribute",
+        .def("_write_attribute",
             (void (*)(Tango::DeviceProxy&, const string &, bopy::object ))
             &PyDeviceProxy::write_attribute,
             ( arg_("self"), arg_("attr_name"), arg_("value") ) )
 
-        .def("write_attribute",
+        .def("_write_attribute",
             (void (*)(Tango::DeviceProxy&, const Tango::AttributeInfo &, bopy::object ))
             &PyDeviceProxy::write_attribute,
             ( arg_("self"), arg_("attr_info"), arg_("value") ) )
 
-        .def("write_attributes",
+        .def("_write_attributes",
             &PyDeviceProxy::write_attributes,
             ( arg_("self"), arg_("name_val") ) )
 
